@@ -17,11 +17,21 @@ void NullProcessInfo::readProcessInfo(int /*pid*/)
 {
 }
 
+bool NullProcessInfo::readProcessName(int /*pid*/)
+{
+    return false;
+}
+
 bool NullProcessInfo::readCurrentDir(int /*pid*/)
 {
     return false;
 }
 
+bool NullProcessInfo::readArguments(int /*pid*/)
+{
+    return false;
+}
+
 void NullProcessInfo::readUserName()
 {
 }
