@@ -11,6 +11,7 @@
 #include "NullProcessInfo.h"
 #include "ProcessInfo.h"
 #include "UnixProcessInfo.h"
+#include "konsoledebug.h"
 
 // Unix
 #ifndef Q_OS_WIN
@@ -26,6 +27,7 @@
 // Qt
 #include <QDir>
 #include <QFileInfo>
+#include <QGuiApplication>
 #include <QHostInfo>
 #include <QStringList>
 #include <QTextStream>
@@ -33,6 +35,7 @@
 
 // KDE
 #include <KConfigGroup>
+#include <KSandbox>
 #include <KSharedConfig>
 #include <KUser>
 
@@ -605,10 +608,20 @@ private:
         return ok;
     }
 
-    QDBusMessage callSmdDBus(const QString &objectPath, const QString &interfaceName, const QString &methodName, const QList<QVariant> &args)
+    QDBusMessage callSmdDBus(const QString &objectPath,
+                             const QString &interfaceName,
+                             const QString &methodName,
+                             const QList<QVariant> &args,
+                             const QString &interfacePrefix = QString{})
     {
         const QString service(QStringLiteral("org.freedesktop.systemd1"));
-        const QString interface(service + QLatin1Char('.') + interfaceName);
+        QString interface;
+        if (interfacePrefix.isEmpty()) {
+            interface = service + u"." + interfaceName;
+        } else {
+            interface = interfacePrefix + u"." + interfaceName;
+        }
+
         auto methodCall = QDBusMessage::createMethodCall(service, objectPath, interface, methodName);
         methodCall.setArguments(args);
 
@@ -624,7 +637,6 @@ private:
         const pid_t selfPid = getpid();
 
         const QString managerObjPath(QStringLiteral("/org/freedesktop/systemd1"));
-        const QString appUnitName(QStringLiteral("app-org.kde.konsole-%1.scope").arg(selfPid));
 
         // check if systemd dbus services exist
         if (!QDBusConnection::sessionBus().interface()->isServiceRegistered(QStringLiteral("org.freedesktop.systemd1"))) {
@@ -634,6 +646,9 @@ private:
         // get current application cgroup path
         const QString oldAppCGroupPath(getProcCGroup(selfPid));
 
+        auto appUnitName = QStringLiteral("app-%1-%2.scope").arg(QGuiApplication::desktopFileName()).arg(selfPid);
+        qCDebug(KonsoleDebug) << "Creating scope" << appUnitName;
+
         // create application unit
         VariantList properties;
         const QList<uint> mainPid({static_cast<quint32>(selfPid)});
@@ -649,13 +664,7 @@ private:
         // get created app cgroup path
         QString newAppCGroupPath = getProcCGroup(selfPid);
         if (newAppCGroupPath == oldAppCGroupPath) {
-            // FIXME, find a better way to do this
-            QThread::msleep(5); // wait for new unit to be created.
-            newAppCGroupPath = getProcCGroup(selfPid);
-
-            if (newAppCGroupPath == oldAppCGroupPath) {
-                return false;
-            }
+            return false;
         }
 
         _createdAppCGroupPath = newAppCGroupPath;
@@ -711,10 +720,41 @@ private:
 
     bool createSystemdUnit(const QString &name, const VariantList &propList)
     {
-        const QList<QVariant> args({name, QStringLiteral("fail"), QVariant::fromValue(propList), QVariant::fromValue(EmptyArray())});
+        using namespace Qt::StringLiterals;
 
-        return callSmdDBus(QStringLiteral("/org/freedesktop/systemd1"), QStringLiteral("Manager"), QStringLiteral("StartTransientUnit"), args).type()
-            != QDBusMessage::ErrorMessage;
+        const QList<QVariant> args({name, u"fail"_s, QVariant::fromValue(propList), QVariant::fromValue(EmptyArray())});
+
+        auto message = callSmdDBus(u"/org/freedesktop/systemd1"_s, u"Manager"_s, u"StartTransientUnit"_s, args);
+        if (message.type() == QDBusMessage::ErrorMessage) {
+            return false;
+        }
+
+        auto reply = QDBusReply<QDBusObjectPath>(message);
+        if (!reply.isValid()) {
+            return false;
+        }
+
+        auto path = reply.value();
+
+        // Wait for the job to finish
+        // We really should be listening for the "JobRemoved" signal of systemd's Manager object,
+        // but that is hard as it makes things async, so instead just wait for the job object to
+        // disappear and if it does, assume the job finished successfully.
+        int tries = 10;
+        bool jobExists = true;
+        while (tries > 0) {
+            const auto jobArgs = QList<QVariant>{u"org.freedesktop.systemd1.Job"_s, u"State"_s};
+            auto jobReply = callSmdDBus(path.path(), u"Properties"_s, u"Get"_s, jobArgs, u"org.freedesktop.DBus"_s);
+            if (jobReply.type() == QDBusMessage::ErrorMessage) {
+                jobExists = false;
+                break;
+            }
+
+            QThread::sleep(std::chrono::milliseconds(10));
+            tries--;
+        }
+
+        return !jobExists;
     }
 
     bool createCGroup(const QString &name, int initialPid)
@@ -748,6 +788,85 @@ private:
 QString LinuxProcessInfo::_createdAppCGroupPath = QString();
 bool LinuxProcessInfo::_cGroupCreationFailed = false;
 
+class FlatpakProcessInfo : public ProcessInfo
+{
+public:
+    FlatpakProcessInfo(int pid, QByteArrayView tty)
+        : ProcessInfo(pid)
+    {
+        QProcess proc;
+        proc.setProgram(QStringLiteral("ps"));
+        proc.setArguments({
+            QStringLiteral("-o"),
+            QStringLiteral("%U\t%p\t%c"),
+            QStringLiteral("--tty"),
+            QString::fromLatin1(tty.data()),
+            QStringLiteral("--no-headers"),
+        });
+
+        KSandbox::startHostProcess(proc, QProcess::ReadOnly);
+
+        proc.waitForStarted();
+        proc.waitForFinished();
+
+        const QString out = QString::fromUtf8(proc.readAllStandardOutput());
+        for (QStringView line : QStringTokenizer(out, u'\n', Qt::SkipEmptyParts)) {
+            using Container = QVarLengthArray<QStringView, 4>;
+            const Container tokens = QStringTokenizer(line, u'\t', Qt::SkipEmptyParts).toContainer<Container>();
+            if (tokens[1].toInt() == pid) {
+                setUserName(tokens[0].trimmed().toString());
+                setPid(tokens[1].trimmed().toInt());
+                setName(tokens[2].trimmed().toString());
+            }
+        }
+    }
+
+    void readProcessInfo(int) override
+    {
+        /* done in ctor */
+    }
+
+    bool readProcessName(int) override
+    {
+        return true;
+    }
+
+    bool readCurrentDir(int) override
+    {
+        bool ok = false;
+        QString pidString = QString::number(pid(&ok));
+        if (!ok) {
+            return false;
+        }
+
+        QProcess proc;
+        proc.setProgram(QStringLiteral("pwdx"));
+        proc.setArguments({pidString});
+        KSandbox::startHostProcess(proc, QProcess::ReadOnly);
+        if (proc.waitForStarted() && proc.waitForFinished()) {
+            proc.setReadChannel(QProcess::StandardOutput);
+            char buffer[4096];
+            int readCount = proc.readLine(buffer, sizeof(buffer));
+            auto line = readCount > 0 ? QByteArrayView(buffer, readCount).trimmed() : QByteArrayView();
+            if (int colon = line.indexOf(": "); colon != -1) {
+                setCurrentDir(QString::fromUtf8(line.mid(colon + 2)));
+                return true;
+            }
+        }
+        return false;
+    }
+
+    bool readArguments(int) override
+    {
+        return false;
+    }
+
+    void readUserName(void) override
+    {
+        /* done in ctor */
+    }
+};
+
 #elif defined(Q_OS_FREEBSD)
 class FreeBSDProcessInfo : public UnixProcessInfo
 {
@@ -1272,11 +1391,19 @@ private:
 };
 #endif
 
-ProcessInfo *ProcessInfo::newInstance(int pid, int sessionPid)
+ProcessInfo *ProcessInfo::newInstance(int pid, int sessionPid, QByteArrayView tty)
 {
     ProcessInfo *info;
 #if defined(Q_OS_LINUX)
-    info = new LinuxProcessInfo(pid, sessionPid);
+    if (KSandbox::isFlatpak()) {
+        if (tty.isEmpty()) {
+            info = new NullProcessInfo(pid);
+        } else {
+            info = new FlatpakProcessInfo(pid, tty);
+        }
+    } else {
+        info = new LinuxProcessInfo(pid, sessionPid);
+    }
 #elif defined(Q_OS_SOLARIS)
     info = new SolarisProcessInfo(pid);
 #elif defined(Q_OS_MACOS)
