@@ -14,7 +14,7 @@
 
 namespace Konsole
 {
-class ColorFilterHotSpot : public RegExpFilterHotSpot
+class KONSOLEPRIVATE_EXPORT ColorFilterHotSpot : public RegExpFilterHotSpot
 {
 public:
     ColorFilterHotSpot(int startLine, int startColumn, int endLine, int endColumn, const QStringList &capturedTexts, const QColor &color);
