@@ -5,7 +5,9 @@
 #ifndef QUICKCOMMANDSWIDGET_H
 #define QUICKCOMMANDSWIDGET_H
 
+#include <QKeySequence>
 #include <QWidget>
+
 #include <memory>
 
 #include "quickcommanddata.h"
@@ -54,6 +56,11 @@ public:
     void setModel(QuickCommandsModel *model);
     void setCurrentController(Konsole::SessionController *controller);
 
+    Q_SIGNAL void quickAccessShortcutChanged(QKeySequence shortcut);
+
+protected:
+    void showEvent(QShowEvent *) override;
+
 private:
     QuickCommandData data() const;
     void prepareEdit();
