@@ -20,10 +20,12 @@
 
 // Qt
 #include <QApplication>
+#include <QByteArray>
 #include <QColor>
 #include <QDir>
 #include <QFile>
 #include <QKeyEvent>
+#include <QRandomGenerator>
 #include <QThread>
 
 // KDE
@@ -34,17 +36,25 @@
 #include <KNotification>
 #include <KProcess>
 #include <KSelectAction>
+#include <KWindowSystem>
 
 #ifndef Q_OS_WIN
 #include <KPtyDevice>
 #endif
 #include <KShell>
 
+// wayland window activation
+#define HAVE_WAYLAND __has_include(<KWaylandExtras>)
+#if HAVE_WAYLAND
+#include <KWaylandExtras>
+#endif
+
 // Konsole
 #if HAVE_DBUS
 #include <sessionadaptor.h>
 #endif
 
+#include "KonsoleSettings.h"
 #include "Pty.h"
 #include "SSHProcessInfo.h"
 #include "SessionController.h"
@@ -76,12 +86,22 @@
 
 using namespace Konsole;
 
-static bool show_disallow_certain_dbus_methods_message = true;
-
 static const int ZMODEM_BUFFER_SIZE = 1048576; // 1 Mb
 
+// compute a securely random cookie used for activationToken
+static QString computeRandomCookie()
+{
+    // get good random data
+    quint32 array[8];
+    QRandomGenerator::global()->fillRange(array);
+
+    // convert to string usable for env var KONSOLE_DBUS_ACTIVATION_COOKIE
+    return QString::fromUtf8(QByteArray(reinterpret_cast<const char *>(array), sizeof(array)).toBase64());
+}
+
 Session::Session(QObject *parent)
     : QObject(parent)
+    , m_activationCookie(computeRandomCookie())
 {
     _uniqueIdentifier = QUuid::createUuid();
 
@@ -268,7 +288,8 @@ void Session::setInitialWorkingDirectory(const QString &dir)
 
 QString Session::currentWorkingDirectory()
 {
-    if (_reportedWorkingUrl.isValid() && _reportedWorkingUrl.isLocalFile() && (_reportedWorkingUrl.host().length() == 0 || _reportedWorkingUrl.host().compare(QSysInfo::machineHostName(), Qt::CaseInsensitive) == 0)) {
+    if (_reportedWorkingUrl.isValid() && _reportedWorkingUrl.isLocalFile()
+        && (_reportedWorkingUrl.host().length() == 0 || _reportedWorkingUrl.host().compare(QSysInfo::machineHostName(), Qt::CaseInsensitive) == 0)) {
         return _reportedWorkingUrl.path();
     }
 
@@ -555,27 +576,36 @@ void Session::run()
 
     addEnvironmentEntry(QStringLiteral("WINDOWID=%1").arg(QString::number(windowId())));
 
+    // env vars we shall not expose e.g. over dbus
+    QStringList secretEnv;
+
 #if HAVE_DBUS
     const QString dbusService = QDBusConnection::sessionBus().baseService();
     addEnvironmentEntry(QStringLiteral("KONSOLE_DBUS_SERVICE=%1").arg(dbusService));
 
     const QString dbusObject = QStringLiteral("/Sessions/%1").arg(QString::number(_sessionId));
     addEnvironmentEntry(QStringLiteral("KONSOLE_DBUS_SESSION=%1").arg(dbusObject));
+
+    // secret cookie to trigger activationToken via dbus
+    secretEnv << QStringLiteral("KONSOLE_DBUS_ACTIVATION_COOKIE=%1").arg(m_activationCookie);
 #endif
 
+    // stuff set via addEnvironmentEntry and the secret parts
+    const QStringList fullEnv = _environment + secretEnv;
+
 #ifndef Q_OS_WIN
     const auto originalEnvironment = _shellProcess->environment();
     _shellProcess->setProgram(exec);
-    _shellProcess->setEnvironment(originalEnvironment + _environment);
+    _shellProcess->setEnvironment(originalEnvironment + fullEnv);
     const auto context = KSandbox::makeHostContext(*_shellProcess);
     arguments = postProcessArgs(context.arguments, arguments);
     _shellProcess->setEnvironment(originalEnvironment);
-    const auto result = _shellProcess->start(context.program, arguments, _environment);
+    const auto result = _shellProcess->start(context.program, arguments, fullEnv);
 #else // Q_OS_WIN
     const auto size = _emulation->imageSize();
     const int lines = size.height();
     const int cols = size.width();
-    int result = _shellProcess->start(exec, arguments, _initialWorkingDir.isEmpty() ? QDir::currentPath() : _initialWorkingDir, _environment, cols, lines);
+    int result = _shellProcess->start(exec, arguments, _initialWorkingDir.isEmpty() ? QDir::currentPath() : _initialWorkingDir, fullEnv, cols, lines);
 #endif
 
     if (result < 0) {
@@ -661,6 +691,15 @@ void Session::setSessionAttribute(int what, const QString &caption)
         return;
     }
 
+    if (what == ResetTextColor || what == ResetBackgroundColor) {
+        QColor color;
+        if (what == ResetTextColor) {
+            Q_EMIT changeForegroundColorRequest(color);
+        } else {
+            Q_EMIT changeBackgroundColorRequest(color);
+        }
+    }
+
     if (modified) {
         Q_EMIT sessionAttributeChanged();
     }
@@ -891,7 +930,7 @@ void Session::sendSignal(int signal)
 
 void Session::reportColor(SessionAttributes r, const QColor &c, uint terminator)
 {
-#define to65k(a) (QStringLiteral("%1").arg(int(((a)*0xFFFF)), 4, 16, QLatin1Char('0')))
+#define to65k(a) (QStringLiteral("%1").arg(int(((a) * 0xFFFF)), 4, 16, QLatin1Char('0')))
     QString msg = QStringLiteral("\033]%1;rgb:").arg(r) + to65k(c.redF()) + QLatin1Char('/') + to65k(c.greenF()) + QLatin1Char('/') + to65k(c.blueF());
 
     // Match termination of OSC reply to termination of OSC request.
@@ -917,11 +956,12 @@ void Session::reportBackgroundColor(const QColor &c, uint terminator)
 bool Session::kill(int signal)
 {
 #ifndef Q_OS_WIN
-    if (processId() <= 0) {
+    const int pid = KSandbox::isFlatpak() ? _shellProcess->flatpakSpawnProcessId() : processId();
+    if (pid <= 0) {
         return false;
     }
 
-    int result = ::kill(processId(), signal);
+    int result = ::kill(pid, signal);
 
     if (result == 0) {
         return _shellProcess->waitForFinished(1000);
@@ -1013,29 +1053,19 @@ void Session::sendTextToTerminal(const QString &text, const QChar &eol) const
 // Only D-Bus calls this function (via SendText or runCommand)
 void Session::sendText(const QString &text) const
 {
-    if (isReadOnly()) {
+    // avoid that we expose this if not wanted by the user
+    if (isCalledViaDbusAndForbidden() || isReadOnly()) {
         return;
     }
 
-#if !REMOVE_SENDTEXT_RUNCOMMAND_DBUS_METHODS
-    if (show_disallow_certain_dbus_methods_message) {
-        KNotification::event(KNotification::Warning,
-                             QStringLiteral("Konsole D-Bus Warning"),
-                             i18n("The D-Bus methods sendText/runCommand were just used.  There are security concerns about allowing these methods to be "
-                                  "public.  If desired, these methods can be changed to internal use only by re-compiling Konsole. <p>This warning will only "
-                                  "show once for this Konsole instance.</p>"));
-
-        show_disallow_certain_dbus_methods_message = false;
-    }
-#endif
-
     _emulation->sendText(text);
 }
 
 // Only D-Bus calls this function
 void Session::runCommand(const QString &command) const
 {
-    if (isReadOnly()) {
+    // avoid that we expose this if not wanted by the user
+    if (isCalledViaDbusAndForbidden() || isReadOnly()) {
         return;
     }
 
@@ -1044,7 +1074,8 @@ void Session::runCommand(const QString &command) const
 
 void Session::sendMouseEvent(int buttons, int column, int line, int eventType)
 {
-    if (isReadOnly()) {
+    // avoid that we expose this if not wanted by the user
+    if (isCalledViaDbusAndForbidden() || isReadOnly()) {
         return;
     }
 
@@ -1115,7 +1146,8 @@ QStringList Session::environment() const
 
 void Session::setEnvironment(const QStringList &environment)
 {
-    if (isReadOnly()) {
+    // avoid that we expose this if not wanted by the user
+    if (isCalledViaDbusAndForbidden() || isReadOnly()) {
         return;
     }
 
@@ -1185,7 +1217,14 @@ void Session::updateSessionProcessInfo()
     // return 0
     if ((_sessionProcessInfo == nullptr) || (processId() != 0 && processId() != _sessionProcessInfo->pid(&ok))) {
         delete _sessionProcessInfo;
-        _sessionProcessInfo = ProcessInfo::newInstance(processId());
+
+        int sessionPid = processId();
+#ifndef Q_OS_WIN
+        _sessionProcessInfo = ProcessInfo::newInstance(sessionPid, -1, _shellProcess->pty()->ttyName());
+#else
+        _sessionProcessInfo = ProcessInfo::newInstance(sessionPid, -1);
+#endif
+
         _sessionProcessInfo->setUserHomeDir();
     }
     _sessionProcessInfo->update();
@@ -1198,7 +1237,11 @@ bool Session::updateForegroundProcessInfo()
     const int foregroundPid = _shellProcess->foregroundProcessGroup();
     if (foregroundPid != _foregroundPid) {
         delete _foregroundProcessInfo;
+#ifndef Q_OS_WIN
+        _foregroundProcessInfo = ProcessInfo::newInstance(foregroundPid, processId(), _shellProcess->pty()->ttyName());
+#else
         _foregroundProcessInfo = ProcessInfo::newInstance(foregroundPid, processId());
+#endif
         _foregroundPid = foregroundPid;
     }
 
@@ -1712,7 +1755,7 @@ void Session::setPreferredSize(const QSize &size)
 
 int Session::processId() const
 {
-    return _shellProcess->processId();
+    return _shellProcess->shellProcessId();
 }
 
 void Session::setTitle(int role, const QString &title)
@@ -2161,4 +2204,69 @@ void Session::runCommandFromLayout(const QString &command) const
     _emulation->sendText(command + QLatin1Char('\n'));
 }
 
+QString Session::activationToken(const QString &cookieForRequest) const
+{
+    // safety check, only work if the caller knows our id
+    // they will read it from the SHELL_SESSION_ID env var inside this session
+    if (cookieForRequest != m_activationCookie) {
+        return {};
+    }
+
+#if HAVE_DBUS && HAVE_WAYLAND
+    // no window active, no token
+    // same if we don't run wayland
+    const auto window = qApp->activeWindow();
+    if (!window || !window->window() || !KWindowSystem::isPlatformWayland()) {
+        return {};
+    }
+
+    // we will respond delayed, as the token needs to arrive
+    Q_ASSERT(calledFromDBus());
+    const auto msg = message();
+    setDelayedReply(true);
+
+    // we need to filter the response with the request serial
+    const int launchedSerial = KWaylandExtras::self()->lastInputSerial(window->window()->windowHandle());
+    connect(
+        KWaylandExtras::self(),
+        &KWaylandExtras::xdgActivationTokenArrived,
+        this,
+        [msg, launchedSerial](int tokenSerial, QString token) {
+            // if wrong token, ignore it, but we must always reply to not stall the caller
+            // we use here a SingleShotConnection, we will just be called once!
+            if (tokenSerial != launchedSerial) {
+                token.clear();
+            }
+            auto reply = msg.createReply(token);
+            QDBusConnection::sessionBus().send(reply);
+        },
+        Qt::SingleShotConnection);
+
+    KWaylandExtras::requestXdgActivationToken(window->window()->windowHandle(), launchedSerial, {});
+
+#endif
+
+    return {};
+}
+
+bool Session::isCalledViaDbusAndForbidden() const
+{
+#if HAVE_DBUS
+    // we can check if this is called via dbus and then consult the config
+    if (calledFromDBus() && !Konsole::KonsoleSettings::enableSecuritySensitiveDBusAPI()) {
+        // trigger error reply
+        setDelayedReply(true);
+        auto reply = message().createErrorReply(QDBusError::AccessDenied, i18n("Security sensitive DBus API is disabled in the settings."));
+        QDBusConnection::sessionBus().send(reply);
+
+        // forbid call
+        return true;
+    }
+    return false;
+#else
+    // no dbus, just allow it
+    return false;
+#endif
+}
+
 #include "moc_Session.cpp"
