@@ -17,8 +17,6 @@
 // TODO: Move the Property away from Profile.h
 #include "profile/Profile.h"
 
-#include "konsolesession_export.h"
-
 class KConfig;
 
 namespace Konsole
@@ -29,7 +27,7 @@ class Profile;
 /**
  * Manages running terminal sessions.
  */
-class KONSOLESESSION_EXPORT SessionManager : public QObject
+class KONSOLEPRIVATE_EXPORT SessionManager : public QObject
 {
     Q_OBJECT
 
