@@ -24,6 +24,7 @@
 #include <QDebug>
 #include <QElapsedTimer>
 #include <QPainter>
+#include <QPainterPath>
 #include <QPen>
 #include <QRect>
 #include <QRegion>
@@ -896,12 +897,11 @@ void TerminalPainter::drawAboveText(QPainter &painter,
                     if (ulColorTable != nullptr && (style[lastX].flags & EF_UNDERLINE_COLOR) != 0) {
                         pen.setColor(ulColorTable[((style[lastX].flags & EF_UNDERLINE_COLOR)) / EF_UNDERLINE_COLOR_1 - 1].color(colorTable));
                     }
-                    int lw = m_parentDisplay->terminalFont()->lineWidth();
+                    const int lw = m_parentDisplay->terminalFont()->lineWidth();
                     qreal y = rect.y() + m_parentDisplay->terminalFont()->fontAscent() + m_parentDisplay->terminalFont()->underlinePos()
                         + m_parentDisplay->terminalFont()->lineSpacing() / static_cast<qreal>(2);
                     if (underline == RE_UNDERLINE_DOUBLE || underline == RE_UNDERLINE_CURL) {
                         y = rect.bottom() - 1;
-                        lw = 1;
                     }
                     pen.setWidth(lw);
                     if (underline == RE_UNDERLINE_DOT) {
@@ -909,17 +909,39 @@ void TerminalPainter::drawAboveText(QPainter &painter,
                     } else if (underline == RE_UNDERLINE_DASH) {
                         pen.setStyle(Qt::DashLine);
                     }
-                    if (underline == RE_UNDERLINE_CURL) {
-                        QVector<qreal> dashes(2, 2);
-                        pen.setDashPattern(dashes);
-                    }
                     painter.setPen(pen);
-                    painter.drawLine(QLineF(rect.x() + fontWidth * startUnderline, y, rect.x() + fontWidth * i - 1, y));
-                    if (underline == RE_UNDERLINE_DOUBLE) {
-                        painter.drawLine(rect.x() + fontWidth * startUnderline, y - 2, rect.x() + fontWidth * i - 1, y - 2);
-                    }
-                    if (underline == RE_UNDERLINE_CURL) {
-                        painter.drawLine(QLineF(rect.x() + fontWidth * startUnderline + 2, y - 1, rect.x() + fontWidth * i - 1, y - 1));
+                    const int x1 = rect.x() + fontWidth * startUnderline;
+                    const int x2 = rect.x() + fontWidth * i - 1;
+                    if (underline != RE_UNDERLINE_CURL)
+                        painter.drawLine(QLineF(x1, y, x2, y));
+                    if (underline == RE_UNDERLINE_DOUBLE || underline == RE_UNDERLINE_CURL) {
+                        const int fontHeight = m_parentDisplay->terminalFont()->fontHeight();
+                        const qreal amplitude = fontHeight / 8;
+                        if (underline == RE_UNDERLINE_DOUBLE) {
+                            if (amplitude)
+                                painter.drawLine(x1, y - amplitude, x2, y - amplitude);
+                        } else {
+                            y = std::max(static_cast<qreal>(0), y - amplitude / 2);
+                            assert(underline == RE_UNDERLINE_CURL);
+                            const int len = x2 - x1;
+                            if (len > 0) {
+                                const qreal desiredWavelength = fontWidth / 1.2;
+                                const int cycles = std::max(1, static_cast<int>(len / desiredWavelength));
+                                const qreal wavelength = static_cast<qreal>(len) / cycles;
+                                const qreal halfWavelength = wavelength / 2.0;
+                                const qreal quarterWavelength = halfWavelength / 2.0;
+                                const qreal threeQuarterWavelength = 3 * quarterWavelength;
+                                QPainterPath segment;
+                                segment.moveTo(0, 0);
+                                segment.quadTo(quarterWavelength, -amplitude, halfWavelength, 0);
+                                segment.quadTo(threeQuarterWavelength, amplitude, wavelength, 0);
+                                QPainterPath path;
+                                path.moveTo(x1, y);
+                                for (int i = 0; i < cycles; ++i)
+                                    path.addPath(segment.translated(x1 + i * wavelength, y));
+                                painter.drawPath(path);
+                            }
+                        }
                     }
 
                     startUnderline = -1;
